/*
 * The print option of the dialing directory.  A carriage return will
 * send the dialing directory to the print spool program, otherwise the
 * selected file will be used.
 */

#define LPRINT "/usr/bin/lprint"

#include <stdio.h>
#include <curses.h>
#include "dial_dir.h"
#include "misc.h"

void
print_dir()
{
	FILE *fp, *popen();
	WINDOW *p_win, *newwin();
	char *file, *get_str_erase(), buf[80];
	int is_printer, i, can;
	void error_win();
	unsigned int sleep();
	
	p_win = newwin(5, 54, 0, 26);

	mvwaddstr(p_win, 2, 3, "Print to: (printer)");
	box(p_win, '|', '-');
	wmove(p_win, 2, 13);
	wrefresh(p_win);

	/*
	 * This is a special version of get_str() that looks at the
	 * first character to see if it should erase the default answer
	 * already on the screen.
	 */
	if ((file = get_str_erase(p_win, 40)) == NULL) {
					/* erase because it overlaps dm_win */
		werase(p_win);
		wrefresh(p_win);
		delwin(p_win);
		return;
	}
	is_printer = 0;
					/* the default (printer) */
	if (*file == '\0') {
		if (!(fp = popen(LPRINT, "w"))) {
			sprintf(buf, "'%s'", LPRINT);
			error_win(0, "Can't open printer device", buf);
			werase(p_win);
			wrefresh(p_win);
			delwin(p_win);
			return;
		}
		is_printer = 1;
	}
					/* the requested file */
	else {
		/*
		 * Check to see if the file already exists (and if we
		 * have write permission too).  Currently only allows
		 * you to bail out or overwrite the file.
		 */
		if (!(can = can_write(file))) {
			sprintf(buf, "'%s'", file);
			error_win(0, "No write permission on file", buf);
			werase(p_win);
			wrefresh(p_win);
			delwin(p_win);
			return;
		}
		if (can == 2) {
			werase(p_win);
			mvwprintw(p_win, 2, 3, "File '%s' already exists!", file);
			beep();
			box(p_win, '|', '-');
			if (!yes_prompt(p_win, 3, 3, A_BOLD, "Overwrite")) {
				werase(p_win);
				wrefresh(p_win);
				delwin(p_win);
				return;
			}
		}
			
		fp = fopen(file, "w");
	}
	
	werase(p_win);
	mvwaddstr(p_win, 2, 13, "Printing pcomm directory");
	box(p_win, '|', '-');
	wrefresh(p_win);

	/*
	 * Only prints up to the end of the physical file, not the entire
	 * structure.  I gave some thought about not printing empty entries,
	 * but...
	 */
	for (i=1; i<=dir->d_entries; i++)
		fprintf(fp, "%4d- %-20.20s %18.18s  %5d-%c-%d-%d  %c  %-14.14s\n",
		 i, dir->name[i], dir->number[i], dir->baud[i], dir->parity[i],
		 dir->dbits[i], dir->sbits[i], dir->duplex[i], dir->index[i]);

	if (is_printer)
		pclose(fp);
	else {
					/* a dramatic delay... */
		sleep(1);
		fclose(fp);
	}

	werase(p_win);
	wrefresh(p_win);
	delwin(p_win);
	return;
}

/*
 * Get a string from a window but erase the line first.
 */

char *
get_str_erase(win, num)
WINDOW *win;
int num;
{
	int count, x, y, done_it;
	char ans;
	static char buf[80];

	done_it = 0;
	count = 0;
	while ((ans = wgetch(win)) != '\r') {
					/* do our own backspace */
		if (ans == 8) {
			if (!count) {
				beep();
				continue;
			}
			count--;
			buf[count] = '\0';
			getyx(win, y, x);
			x--;
			wmove(win, y, x);
			waddch(win, ' ');
			wmove(win, y, x);
			wrefresh(win);
			continue;
		}
					/* exceeded the max ? */
		if (count == num) {
			beep();
			continue;
		}
					/* an ESC anywhere in the string */
		if (ans == 27)
			return(NULL);
					/* erase the default answer */
		if (!done_it) {
			waddstr(win, "         ");
			wmove(win, 2, 13);
			wrefresh(win);
			done_it = 1;
		}

		buf[count] = ans;
		waddch(win, ans);
		wrefresh(win);
		count++;
	}
	buf[count] = '\0';
	return(buf);
}
