/* These functions are designed for Microsoft or Lattice C.  Some of them
have their germ in "The C Programming Language," but most have been
designed to fit a need for me, and then found usefulness over and over. 
I hope they will be as useful to others as they have been to me.
By Dr. Jim Conn, Frankfort, KY  */
#include "stdio.h"
#include "ctype.h"
/*convert string to lower case*/
downstr(b)
char *b;
{
	while(*b != '\0') {
		*b = tolower(*b);
		b++;
	}
}
/*get a line of data from file fi, terminated by a newline, into array s */
/*does not include the newline*/
getl(fi, s)
char s[];
FILE *fi;
{
	int c, i;
	i = 0;
	while ((c = fgetc(fi)) != -1 && (c & 127) != '\n') {
		c = c & 127;
		s[i++] = c;
	}
	s[i] = '\0';
	return(i);
}
/* convert LONG n to a string pointed to by s */
ltoa(n, s)
char *s;
long n;
{
	int i;
	long sign;
	if ((sign = n) < 0L)
		n = -n;
	i = 0;
	do {
		s[i++] = n % 10L + '0';
	} while ((n /= 10L) > 0L);
	if (sign < 0L)
		s[i++] = '-';
	s[i] = '\0';
	reverse(s);
}
/* concatenate a max of len chars from the beginning of string src 
to the end of string dest */
char *strncat(dest, src, len)
register char *dest, *src;
{
        char *sav;

        sav = dest;
        while (*dest)
                ++dest;
        do {
                if (len-- == 0) {
                        *dest = 0;
                        break;
                }
        } while (*dest++ = *src++);
        return sav;
}
/* compare len chars from the beginning of string b to string a */
strncmp(a, b, len)
register char *a, *b;
{
        int i = 0;

        while (len-- && (i = *a - *b++) == 0 && *a++)
                ;
        return i;
}
/* copy n chars from string s2 to string s1 */
strncpy(s1, s2, n)
char *s1, *s2;
unsigned n;
{
	unsigned i;
	i=0;
	while(i < n && s2[i] != '\0')
		{
		s1[i] = s2[i];
		++i;
		}
	while(i < n)
		{
		s1[i] = '\0';
		++i;
		}
	s1[i] = '\0';
}
/*convert string b to uppercase */
upstr(b)
char *b;
{
	while(*b != '\0') {
		*b = toupper(*b);
		b++;
	}
}
/*return hashvalue of string in s, bounded by hashsize */
hash(s, hashsize)
char *s;
int hashsize;
{
	int hashval;
	for (hashval = 0; *s != '\0';)
		hashval += *s++;
	return(hashval % hashsize);
}
/* reverse string s */
reverse(s)
char s[];
{
	int c, i, j;
	for (i = 0, j = strlen(s)-1; i < j; i++, j--) {
		c = s[i];
		s[i] = s[j];
		s[j] = c;
	}
}
instr(s, t)	/* return index of t in s, -1 if none */
char s[], t[];
{
	int i, j, k;
	for (i = 0; s[i] != '\0'; i++) {
		for (j=i, k=0; t[k] != '\0' && s[j] == t[k]; j++, k++)
			;
		if (t[k] == '\0')
			return(++i);
		}
		return(-1);
}
midstr(x, i, j, buf)
int i, j;
char *x, *buf;
{

/*this function returns a string of length j characters
from string x beginning with character number i.
if length of x is less than i, returns null string.
if length of x from i pos. is less than j characters, 
returns all characters to the right of i position.
similar to MID$(X$, I, J) in MBASIC. */

	int l;
	if(i < 1)
		i = 1;
	if((strlen(x)) < i) {
		buf[0] = '\0';
		return(0);
	}
	x = x+i-1;
	if((strlen(x)) < j) {
		strcpy(buf, x);
		return(0);
	}
	for(l = 0; l < j; l++)
		buf[l] = x[l];
	buf[l] = '\0';
}
leftstr(x, i, buf)
int i;
char *x, *buf;
{

/*this function returns a string of length j characters
from the left side of string x.
if i = 0, returns null string.
if i is greater than the length of x, returns x.
similar to LEFT$(X$, I) in MBASIC. */

	int l;
	if(i == 0) {
		buf[0] = '\0';
		return(0);
	}
	if(i > (strlen(x))) {
		strcpy(buf, x);
		return(0);
	}
	for(l = 0; l < i; l++)
		buf[l] = x[l];
	buf[l] = '\0';
}
rightstr(x, i, buf)
int i;
char *x, *buf;
{

/*this function returns a string of length i characters
from the right side of string x.
if i = 0, returns null string.
if i is greater than the length of x, returns x.
similar to  RIGHT$(X$, I) in MBASIC. */

	if(i == 0) {
		buf[0] = '\0';
		return(0);
	}
	if(i > (strlen(x))) {
		strcpy(buf, x);
		return(0);
	}
	x += ((strlen(x)) - i);
	strcpy(buf, x);
	return(0);
}
/* get a line of chars from file fi into s, with maximum of lim chars */
/* if lim is not reached, will return a newline at the end */
getline(fi, s, lim)
char s[];
int lim;
unsigned fi;
{
	int c, i;
	i = 0;
	while (--lim > 0 && (c = fgetc(fi)) != -1 && (c & 127) != '\n') {
		c = c & 127;
		s[i++] = c;
	}
	if ((c & 127) == '\n')
		s[i++] = c & 127;
	s[i] = '\0';
	return(i);
}
/* saves string s, and returns a pointer to the location */
char *strsave(s)
char *s;
{
	char *p, *malloc();
	if ((p = malloc(strlen(s)+1)) != 0)
		strcpy(p,s);
	return(p);
}
/* convert ascii string cp to an int, and returns it */
atoi(cp)
register char *cp;
{
	register unsigned i;
	register sign;

	sign = 0;
	if ( *cp == '-' ) {
		++cp;
		sign = 1;
	} else if ( *cp == '+' )
		++cp;

	for ( i = 0 ; isdigit(*cp) ; )
		i = i*10 + *cp++ - '0';
	return sign ? -i : i;
}
/* converts ascii string cp to a long number, and returns it */
long atol(cp)
register char *cp;
{
	long n;
	register sign;

	sign = 0;
	if ( *cp == '-' ) {
		++cp;
		sign = 1;
	} else if ( *cp == '+' )
		++cp;

	for ( n = 0 ; isdigit(*cp) ; )
		n = n*10 + *cp++ - '0';
	return sign ? -n : n;
}
/* convert ascii string s to a double precision number, & return it. */
double atof(s)
char s[];
{
	double val, power;
	int i, sign;
	for(i=0; s[i]==' ' || s[i]=='\n' || s[i]=='\t'; i++)
		;
	sign = 1;
	if (s[i] == '+' || s[i] == '-')
		sign = (s[i++]=='+') ? 1 : -1;
	for(val = 0; s[i] >= '0' && s[i] <= '9'; i++)
		val = 10 * val + s[i] - '0';
	if (s[i] == '.')
		i++;
	for (power = 1; s[i] >= '0' && s[i] <= '9'; i++) {
		val = 10 * val + s[i] - '0';
		power *= 10;
	}
	return(sign * val / power);
}
/* convert int n to a string, and return in s */
itoa(n, s)
char s[];
int n;
{
	int i, sign;
	if ((sign = n) < 0)
		n = -n;
	i = 0;
	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	reverse(s);
}
/* get the next ascii number string from file fi, and return it in w */
getn(fi, w)
char *w;
unsigned fi;
{
	int c, t;
	c = fgetc(fi);
	while(c != EOF && (isdigit(c) == 0))
		c = fgetc(fi);
	if(c == EOF)
		return(EOF);
	while(c != EOF)
		{
		c &= 127;
		if(isdigit(c))
			*w++ = c;
		else
			{
			*w = '\0';
			return(1);
			}
		c = fgetc(fi);
		}
	return(EOF);
}
/* this function is handy for extracting the value of a range of bit
positions in a number, and finds good use when doing system calls
where certain bit positions have value. It returns an int value of
the bits.  NOTE: get n bits from x, beginning at position p, as per
K & R book, p. 45; but the thing is, it gets them from the position p
DOWN! So, to get bits 5, 6 & 7 from number x, specify as follows:
valbits = getbits(x, 7, 3);  See what I mean? */
getbits(x, p, n)
unsigned x, p, n;
{
	return((x >> (p+1-n)) & ~(~0 << n));
}
/* This function returns a paragraph of text from file fi, in string s.
A paragraph is defined as a string of text terminated by a doublespace.
(Two newlines). */
getpgrf(fi, s)
char *s;
unsigned fi;
{
	int d, c, i;
	i = 0;
	while ((c = fgetc(fi)) != EOF) 
		{
		s[i++] = (c & 127);
		if((d = fgetc(fi)) != EOF)
			if((c == '\n') && ((d & 127) == '\n'))
				break;
		else
			ungetc(d, fi);
		}
	s[i] = '\0';
	if(c == EOF || d == EOF)
		return(EOF);
	else
		return(i);
}
/* have fun!*/
